# -*- coding: utf-8 -*-

# Implementation of a Modified Preorder Tree Traversal algorithm
# Theory taken from http://www.sitepoint.com/print/hierarchical-data-database
# More examples at: http://code.djangoproject.com/wiki/ModifiedPreorderTreeTraversal

from django.core import meta

from django.models import auth, core

class Node(meta.Model):
    content_type = meta.ForeignKey(core.ContentType)
    object_id = meta.IntegerField(_('Object ID'), db_index = True)
    
    body = meta.TextField()

    # To create categories for a website menu, we won't probably need to record
    # what user created the node. But if nodes are used for threaded comments
    # or forums posts, it's a good idea to have.
    user = meta.ForeignKey(auth.User, verbose_name = _('User'))
    pub_date = meta.DateTimeField(_('Published date'), auto_now_add = True)
    
    lft = meta.PositiveIntegerField(db_index = True)
    rght = meta.PositiveIntegerField()
    
    def _pre_save(self):
        # Let's check if it's a new root node or a sub-node, because they require
        # different update techniques
        # The image on SitePoint's article really helps to understand why, btw :)
        sub_node_of = self.get_content_type()
        # If it's a sub-node, we need to update the whole tree to the right of it.
        # And change self.content_type to where the root node is attached.
        content_object = self.get_content_object()
        if type(self) == type(content_object):
            cursor = db.cursor()
            
            target_rght = content_object.rght - 1

            cursor.execute("UPDATE `mptt_nodes` " \
                           "SET `rght` = `rght` + 2 " \
                           "WHERE object_id = '%i' "
                           "AND `rght` > %i" % (content_object.object_id, int(target_rght)))

            cursor.execute("UPDATE `mptt_nodes` " \
                           "SET `lft` = `lft` + 2 " \
                           "WHERE object_id = '%i' "
                           "AND `lft` > %i" % (content_object.object_id, int(target_rght)))

            db.commit()
            
            self.content_type = content_object.content_type
            self.object_id = content_object.object_id
            self.lft = target_rght + 1
            self.rght = target_rght + 2
        # If it's a new root, it's a slightly different insert, to accomodate the new node.
        else:
            # If there are zero nodes for this content object so far, we will just
            # insert with lft = 1 and rght = 2
            cursor = db.cursor()
            cursor.execute("SELECT MAX(`rght`) FROM `mptt_nodes` WHERE object_id = '%s'", (self.object_id,))
            row = cursor.fetchone()
            
            current_max_rght = row[0]
            
            if current_max_rght is None:
                self.lft = 1
                self.right = 2
            # But if there are already nodes attached to the object, we will put
            # this new one at the top level (to the right of the 'tree')
            else:
                self.lft = current_max_rght + 1
                self.rght = current_max_rght + 2
        

    def __repr__(self):
        return self.body[:50]
    
    def get_absolute_url(self):
        return self.get_content_object().get_absolute_url() + "#n" + str(self.id)

    def get_content_object(self):
        from django.core.exceptions import ObjectDoesNotExist
        try:
            return self.get_content_type().get_object_for_this_type(pk = self.object_id)
        except ObjectDoesNotExist:
            return None
            
    class META:
        admin = meta.Admin(
            list_display = ('content_type', 'get_content_object',),
        )
